@@ -3,6 +3,8 @@
 #' \itemize{
 #' \item{\code{cro_mean}, \code{cro_sum}, \code{cro_median}}{ calculate 
 #' mean/sum/median by groups. NA's are always omitted.}
+#' \item{\code{cro_mean_sd_n}}{ calculates mean, standard deviation and N
+#' simultaneously. Mainly intended for usage with \link{significance_means}.}
 #' \item{\code{cro_pearson}, \code{cro_spearman}}{ calculate correlation of 
 #' first variable in each data.frame in \code{cell_vars} with other variables. 
 #' NA's are removed pairwise.}
@@ -16,18 +18,20 @@
 #' \code{cell_vars} as a whole. So \code{cro_fun(iris[, -5], iris$Species, fun =
 #' mean)} gives the same result as \code{cro_fun_df(iris[, -5], iris$Species, 
 #' fun = colMeans)}. For \code{cro_fun_df} names of \code{cell_vars} will 
-#' converted to labels if they are available before \code{fun} will be applied.
-#' Generally it is recommended that \code{fun} will always return object of the
+#' converted to labels if they are available before the \code{fun} will be applied. 
+#' Generally it is recommended that \code{fun} will always return object of the 
 #' same form. Row names/vector names of \code{fun} result will appear in the row
-#' labels of the table and column names/names of list will appear in the column
-#' labels.}
+#' labels of the table and column names/names of list will appear in the column 
+#' labels. If your \code{fun} returns data.frame/matrix/list with element named
+#' 'row_labels' then this element will be used as row labels. And it will have
+#' precedence over rownames.}
 #' \item{\code{combine_functions}}{ is auxiliary function for combining several 
 #' functions into one function for usage with \code{cro_fun}/\code{cro_fun_df}.
 #' Names of arguments will be used as statistic labels. By default, results of
 #' each function are combined with \link{c}. But you can provide your own method
 #' function with \code{method} argument. It will be applied as in the expression
 #' \code{do.call(method, list_of_functions_results)}. Particular useful method
-#' is \code{list}. When it used statistic labels will appear in the column
+#' is \code{list}. When it used then statistic labels will appear in the column
 #' labels. See examples. Also you may be interested in \code{data.frame}, 
 #' \code{rbind}, \code{cbind} methods.}}
 #' 
@@ -49,12 +53,29 @@
 #'   \link[data.table]{data.table} with all names converted to variable labels
 #'   (if labels exists). So it is not recommended to rely on original variables
 #'   names in your \code{fun}.
-#' @param ... further arguments for \code{fun}  in
+#' @param ... further arguments for \code{fun}  in 
 #'   \code{cro_fun}/\code{cro_fun_df} or functions for \code{combine_functions}.
+#'   Ignored in \code{cro_fun}/\code{cro_fun_df} if \code{unsafe} is TRUE.
+#' @param weighted_valid_n logical. Sould we show weighted valid N in
+#'   \code{cro_mean_sd_n}? By default it is FALSE.
+#' @param labels character vector of length 3. Labels for mean, standard
+#'   deviation and valid N in \code{cro_mean_sd_n}.
 #' @param method function which will combine results of multiple functions in
 #'   \code{combine_functions}. It will be applied as in the expression 
 #'   \code{do.call(method, list_of_functions_results)}. By default it is
 #'   \code{c}.
+#' @param unsafe logical/character If not FALSE than \code{fun} will be 
+#'   evaluated as is. It can lead to significant increase in the performance. 
+#'   But there are some limitations. For \code{cro_fun} it means that your 
+#'   function \code{fun} should return vector. If length of this vector is
+#'   greater than one than you should provide with \code{unsafe} argument vector
+#'   of unique labels for each element of this vector. There will be no attempts
+#'   to automatically make labels for the results of \code{fun}. For 
+#'   \code{cro_fun_df} your function should return vector or list/data.frame
+#'   (optionally with 'row_labels' element - statistic labels). If \code{unsafe}
+#'   is TRUE or not logical then further arguments (\code{...}) for \code{fun}
+#'   will be ignored.
+#'  
 #'
 #' @return object of class 'etable'. Basically it's a data.frame but class
 #'   is needed for custom methods.
@@ -151,10 +172,11 @@ cro_fun = function(cell_vars,
                    weight = NULL,
                    subgroup = NULL,
                    fun, 
-                   ...){
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+                   ...,
+                   unsafe = FALSE){
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
     
     cell_vars = test_for_null_and_make_dataframe(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
@@ -165,7 +187,7 @@ cro_fun = function(cell_vars,
         stopif(!("weight" %in% names(formals(fun))),
                "`weight` is provided but `fun` doesn't have formal `weight` argument.")
     }
-    fun = make_function_for_cro(fun, ..., need_weight = !is.null(weight))
+    
     
     row_vars = flat_list(multiples_to_single_columns_with_dummy_encoding(row_vars), flat_df = TRUE)
     col_vars = flat_list(multiples_to_single_columns_with_dummy_encoding(col_vars), flat_df = TRUE)
@@ -173,9 +195,12 @@ cro_fun = function(cell_vars,
     
     check_sizes("'cro_fun'", cell_vars, row_vars, col_vars, weight, subgroup)
     
-    cell_vars = make_labels_from_names(cell_vars)
-    
-    
+    if(is.logical(unsafe) && !unsafe){
+        cell_vars = make_labels_from_names(cell_vars)
+        fun = make_function_for_cro(fun, ..., need_weight = !is.null(weight))
+    } else {
+        cell_vars = unlab(names2labels(cell_vars))
+    }
     
     res = lapply(row_vars, function(each_row_var){
         all_col_vars = lapply(col_vars, function(each_col_var){
@@ -184,7 +209,8 @@ cro_fun = function(cell_vars,
                                            col_var = each_col_var, 
                                            weight = weight,
                                            subgroup = subgroup,
-                                           fun = fun
+                                           fun = fun,
+                                           use_lapply = unsafe
             )    
             row_var_lab = var_lab(dtable[["..row_var__"]])
             col_var_lab = var_lab(dtable[["..col_var__"]])
@@ -206,17 +232,19 @@ cro_fun = function(cell_vars,
 }
 
 
+
 ### compute statistics for single row_var and single col_var
 elementary_cro_fun_df = function(cell_var, 
                                  col_var, 
                                  weight,
                                  fun,
                                  row_var,
-                          subgroup
+                          subgroup, 
+                          use_lapply = FALSE
                           ){
     # to pass CRAN check
     ..weight__ = NULL
-    
+    row_labels = NULL
     ### calculate vector of valid cases
 
     valid = valid(col_var) & valid(row_var) & if_null(subgroup, TRUE)
@@ -262,10 +290,37 @@ elementary_cro_fun_df = function(cell_var,
 
     # statistics
     by_string = "..row_var__,..col_var__"
-    if(is.null(weight)){
-        dtable = raw_data[ , fun(.SD), by = by_string]
+    if(is.logical(use_lapply) && !use_lapply){
+        if(is.null(weight)){
+            dtable = raw_data[ , fun(.SD), by = by_string]
+        } else {
+            dtable = raw_data[ , fun(.SD, weight = ..weight__), by = by_string, .SDcols = -"..weight__"]
+        }
     } else {
-        dtable = raw_data[ , fun(.SD, weight = ..weight__), by = by_string, .SDcols = -"..weight__"]
+        if(is.null(weight)){
+            dtable = raw_data[ , lapply(.SD, fun), by = by_string]
+        } else {
+            dtable = raw_data[ , lapply(.SD[, -"..weight__"], fun, weight = ..weight__), by = by_string]
+        }
+        dtable = melt(dtable, id.vars = c("..row_var__", "..col_var__"), 
+                      variable.name = "row_labels",
+                      na.rm = FALSE, variable.factor = TRUE, 
+                      value.factor = FALSE, 
+                      verbose = FALSE)
+        if(!isTRUE(use_lapply)){
+            labels_length = length(use_lapply)
+            stopif(labels_length==0 || (NROW(dtable) %% labels_length)!=0, 
+                   "'cro_fun'/'cro_fun_df' - length of labels provided with 'unsafe' is incorrect: ", 
+                   labels_length)
+            stopif(anyDuplicated(use_lapply),  "'cro_fun'/'cro_fun_df' - duplicated labels provided with 'unsafe': ", 
+                   paste(unique(use_lapply[duplicated(use_lapply)])), collapse = ", ")
+            dtable[ , row_labels := paste(row_labels, use_lapply, sep = "|")]
+        }
+        
+    }    
+    if(("row_labels" %in% colnames(dtable)) && !is.factor(dtable[["row_labels"]])){ 
+        dtable[ , row_labels := fctr(row_labels, levels = unique(row_labels),
+                          prepend_var_lab = FALSE)]
     }
     dtable
 }    
@@ -307,17 +362,25 @@ make_function_for_cro_df = function(fun, ..., need_weight = TRUE){
         function(x, weight = weight){
             res = fun(x, ..., weight = weight)
             res = make_dataframe_with_row_labels(res)
-            # we need convert to factor to keep order of row_labels
-            as.list(res[, row_labels := fctr(row_labels, levels = unique(row_labels),
-                                             prepend_var_lab = FALSE)])
+            # we need convert to factor to keep all row_labels as levels in case of aggregation 
+            # of data with zero rows
+            if(nrow(x)==0){
+                res$row_labels = fctr(res[["row_labels"]], levels = unique(res[["row_labels"]]),
+                                                 prepend_var_lab = FALSE)
+            }
+            res
         }
     } else {
         function(x){
             res = fun(x, ...)
             res = make_dataframe_with_row_labels(res)
-            # we need convert to factor to keep order of row_labels
-            as.list(res[, row_labels := fctr(row_labels, levels = unique(row_labels),
-                                             prepend_var_lab = FALSE)])
+            # we need convert to factor to keep all row_labels as levels in case of aggregation 
+            # of data with zero rows
+            if(nrow(x)==0){
+                res$row_labels = fctr(res[["row_labels"]], levels = unique(res[["row_labels"]]),
+                                      prepend_var_lab = FALSE)
+            }
+            res
         }        
     }
 }
@@ -339,11 +402,15 @@ make_function_for_cro = function(fun, ..., need_weight = TRUE){
                 }
                 each_res
             })
-            res = rbindlist(res, use.names = TRUE, fill = TRUE)
-            # we need convert to factor to keep order of row_labels
-            res[, row_labels := make_items_unique(row_labels)]
-            as.list(res[, row_labels := fctr(row_labels, levels = unique(row_labels),
-                                             prepend_var_lab = FALSE)])
+            res = as.dtfrm(rbindlist(res, use.names = TRUE, fill = TRUE))
+            # we need convert to factor to keep all row_labels as levels in case of aggregation 
+            # of data with zero rows
+            res$row_labels = make_items_unique(res[["row_labels"]])
+            if(nrow(x)==0){
+                res$row_labels = fctr(res[["row_labels"]], levels = unique(res[["row_labels"]]),
+                                      prepend_var_lab = FALSE)
+            }
+            res
         }
     } else {
         function(x){
@@ -356,11 +423,15 @@ make_function_for_cro = function(fun, ..., need_weight = TRUE){
                 }
                 each_res
             })
-            res = rbindlist(res, use.names = TRUE, fill = TRUE)
-            # we need convert to factor to keep order of row_labels
-            res[, row_labels := make_items_unique(row_labels)]
-            as.list(res[, row_labels := fctr(row_labels, levels = unique(row_labels),
-                                             prepend_var_lab = FALSE)])
+            res = as.dtfrm(rbindlist(res, use.names = TRUE, fill = TRUE))
+            # we need convert to factor to keep all row_labels as levels in case of aggregation 
+            # of data with zero rows
+            res$row_labels = make_items_unique(res[["row_labels"]])
+            if(nrow(x)==0){
+                res$row_labels = fctr(res[["row_labels"]], levels = unique(res[["row_labels"]]),
+                                      prepend_var_lab = FALSE)
+            }
+            res
         }        
     }
 }
@@ -368,6 +439,7 @@ make_function_for_cro = function(fun, ..., need_weight = TRUE){
 #############
 make_dataframe_with_row_labels = function(res){
     if(is.table(res)){
+        ### block for 'summary' and 'table'
         dm_names = dimnames(res)
         if(is.null(dm_names)){
             dm_names[[1]] = names(res)
@@ -381,26 +453,32 @@ make_dataframe_with_row_labels = function(res){
         if(length(dm_names)>1){
             new_df = setNames(new_df, dm_names[[2]])
         } else {
-            new_df = setNames(dtfrm(new_df), rep("|", NCOL(new_df)))
+            new_df = setNames(new_df, rep("|", NCOL(new_df)))
         }
         res = new_df
     } else {
+        ####
         if(is.matrix(res) || is_list(res)) res = as.dtfrm(res)
+        ####
         if(is.data.frame(res)) {
-            if("row_labels" %in% colnames(res)){
-                row_labels = res[["row_labels"]]    
-                res[["row_labels"]] = NULL
+            if("row_labels" %in% names(res)){
+                res[["row_labels"]] = make_items_unique(res[["row_labels"]])
             } else {
                 row_labels = rownames(res)
                 if(!is.null(row_labels) && length(row_labels)==1 && row_labels[1]==1){
                     row_labels = ""
                 }
+                res[["row_labels"]] = make_items_unique(row_labels)
             }
+            return(res)
         } else {
+        #### usual vectors and unknowns    
             row_labels = names(res)
             res = setNames(dtfrm(res), rep("|", NCOL(res)))
         } 
     }
+    
+    #######
     if(is.null(row_labels)){
         if(nrow(res)>1){
             row_labels = seq_len(nrow(res)) 
@@ -409,8 +487,8 @@ make_dataframe_with_row_labels = function(res){
         }
     } 
     row_labels = make_items_unique(as.character(row_labels))
-    data.table(row_labels = row_labels, res)
-
+    res[["row_labels"]] = row_labels
+    res
 }
 
 
@@ -426,10 +504,11 @@ cro_fun_df = function(cell_vars,
                       weight = NULL,
                       subgroup = NULL,
                       fun, 
-                      ...){
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+                      ...,
+                      unsafe = FALSE){
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
 
     cell_vars = test_for_null_and_make_list(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
@@ -440,8 +519,9 @@ cro_fun_df = function(cell_vars,
         stopif(!("weight" %in% names(formals(fun))),
                "`weight` is provided but `fun` doesn't have formal `weight` argument.")
     }
-    fun = make_function_for_cro_df(fun, ..., need_weight = !is.null(weight))
-    
+    if(is.logical(unsafe) && !unsafe) {
+        fun = make_function_for_cro_df(fun, ..., need_weight = !is.null(weight))
+    }
     cell_vars = make_labels_from_names(cell_vars)
     cell_vars = flat_list(cell_vars, flat_df = FALSE)
     row_vars = flat_list(multiples_to_single_columns_with_dummy_encoding(row_vars), flat_df = TRUE)
@@ -481,8 +561,12 @@ cro_fun_df = function(cell_vars,
     res
 }
 
+
+
+
 ######################################################
 
+
 #' @export
 #' @rdname cro_fun
 cro_mean = function(cell_vars, 
@@ -492,28 +576,65 @@ cro_mean = function(cell_vars,
                     subgroup = NULL
 ){
     
-    fun = function(x, weight = NULL){
-        res = vapply(x, FUN = w_mean, FUN.VALUE = numeric(1), weight = weight, USE.NAMES = FALSE)
-        list(row_labels = names(x), "|" = res)
-    }
-    
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
     
     cell_vars = test_for_null_and_make_dataframe(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
     col_vars = test_for_null_and_make_list(col_vars, str_col_vars)
     
-    cro_fun_df(cell_vars = cell_vars, 
+    cro_fun(cell_vars = cell_vars, 
                col_vars = col_vars, 
                row_vars = row_vars, 
                weight = weight,
                subgroup = subgroup,
-               fun = fun
+               fun = w_mean,
+               unsafe = TRUE
     )
 }
 
+
+#' @export
+#' @rdname cro_fun
+cro_mean_sd_n = function(cell_vars, 
+                    col_vars = total(), 
+                    row_vars = total(label = ""),
+                    weight = NULL,
+                    subgroup = NULL,
+                    weighted_valid_n = FALSE,
+                    labels = c("Mean", "Std. dev.", 
+                               ifelse(weighted_valid_n, "Valid N", "Unw. valid N")) 
+                    
+){
+    
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
+    
+    cell_vars = test_for_null_and_make_dataframe(cell_vars, str_cell_vars)
+    row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
+    col_vars = test_for_null_and_make_list(col_vars, str_col_vars)
+    if(weighted_valid_n){
+        fun = function(x, weight = NULL) {
+            c(w_mean(x, weight), w_sd(x, weight), valid_n(x, weight))
+        }
+    } else {
+        fun = function(x, weight = NULL) {
+            c(w_mean(x, weight), w_sd(x, weight), unweighted_valid_n(x, weight))
+        }
+    }
+    cro_fun(cell_vars = cell_vars, 
+            col_vars = col_vars, 
+            row_vars = row_vars, 
+            weight = weight,
+            subgroup = subgroup,
+            fun = fun,
+            unsafe = labels
+    )
+}
+
+
 #' @export
 #' @rdname cro_fun
 cro_sum = function(cell_vars, 
@@ -522,25 +643,22 @@ cro_sum = function(cell_vars,
                    weight = NULL,
                    subgroup = NULL
 ){
-    fun = function(x, weight = NULL){
-        res = vapply(x, FUN = w_sum, FUN.VALUE = numeric(1), weight = weight, USE.NAMES = FALSE)
-        list(row_labels = names(x), "|" = res)
-    }
-    
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
     
     cell_vars = test_for_null_and_make_dataframe(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
     col_vars = test_for_null_and_make_list(col_vars, str_col_vars)
     
-    cro_fun_df(cell_vars = cell_vars, 
-               col_vars = col_vars, 
-               row_vars = row_vars, 
-               weight = weight,
-               subgroup = subgroup,
-               fun = fun
+    cro_fun(cell_vars = cell_vars, 
+                         col_vars = col_vars, 
+                         row_vars = row_vars, 
+                         weight = weight,
+                         subgroup = subgroup,
+                         fun = w_sum,
+                         unsafe = TRUE
     )
 }
 
@@ -552,25 +670,22 @@ cro_median = function(cell_vars,
                       weight = NULL,
                       subgroup = NULL
 ){
-    fun = function(x, weight = NULL){
-        res = vapply(x, FUN = w_median, FUN.VALUE = numeric(1), weight = weight, USE.NAMES = FALSE)
-        list(row_labels = names(x), "|" = res)
-    }
-    
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
     
     cell_vars = test_for_null_and_make_dataframe(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
     col_vars = test_for_null_and_make_list(col_vars, str_col_vars)
     
-    cro_fun_df(cell_vars = cell_vars, 
-               col_vars = col_vars, 
-               row_vars = row_vars, 
-               weight = weight,
-               subgroup = subgroup,
-               fun = fun
+    cro_fun(cell_vars = cell_vars, 
+                         col_vars = col_vars, 
+                         row_vars = row_vars, 
+                         weight = weight,
+                         subgroup = subgroup,
+                         fun = w_median,
+                         unsafe = TRUE
     )
 }
 
@@ -591,9 +706,9 @@ cro_pearson = function(cell_vars,
         w_pearson(x, weight = weight)[ , 1]
     }    
     
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
     
     cell_vars = test_for_null_and_make_list(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
@@ -621,9 +736,9 @@ cro_spearman = function(cell_vars,
         w_spearman(x, weight = weight)[ , 1]
     }    
     
-    str_cell_vars = deparse(substitute(cell_vars))
-    str_row_vars = deparse(substitute(row_vars))
-    str_col_vars = deparse(substitute(col_vars))
+    str_cell_vars = expr_to_character(substitute(cell_vars))
+    str_row_vars = expr_to_character(substitute(row_vars))
+    str_col_vars = expr_to_character(substitute(col_vars))
     
     cell_vars = test_for_null_and_make_list(cell_vars, str_cell_vars)
     row_vars = test_for_null_and_make_list(row_vars, str_row_vars)
@@ -645,7 +760,7 @@ cro_spearman = function(cell_vars,
 #' @rdname cro_fun
 combine_functions = function(..., method = c){
     method = match.fun(method)
-    possible_names = unlist(lapply(as.list(substitute(list(...)))[-1], deparse))
+    possible_names = unlist(lapply(as.list(substitute(list(...)))[-1], expr_to_character))
     args = list(...)
     arg_names =names(args)
     if(is.null(arg_names)) {
@@ -660,8 +775,11 @@ combine_functions = function(..., method = c){
                        paste0("`weight` is provided but function`", names(args)[each],
                               "` doesn't have formal `weight` argument."))
             }    
+            res = lapply(args, function(f) f(x, weight = weight))
+        } else {
+            res = lapply(args, function(f) f(x))    
         }
-        res = lapply(args, function(f) f(x))
+        
         do.call(method, res)
     }
 }
